from argparse import ArgumentParser
from functools import partial
from pandas import DataFrame
import sys
import warnings

from tabulate import tabulate

from opa_tools.request import (
    create_template,
    generate_requests,
    get_request,
    get_request_identifiers
)

def _get_parser():
    parser = ArgumentParser()
    subparsers = parser.add_subparsers(title="Service", dest="service")

    parser_show = subparsers.add_parser("show")
    _add_table_arguments(parser_show)

    parser_ls = subparsers.add_parser("ls")
    parser_ls.add_argument(
        "identifier",
        nargs="?",
        default="*",
        help="Month of request columns, as YYMM, YYMM/<REQID> or <REQID>",
    )
    parser_ls.add_argument(
        "-s", "--summarize",
        action="store_true",
        help="Collapse subrequests, do show them as a single request.",
    )

    parser_col = subparsers.add_parser("col")

    parser_gen = subparsers.add_parser("gen")
    parser_gen.add_argument("identifier", type=str, action="store")

    parser_gen = subparsers.add_parser("template")
    parser_gen.add_argument("dst", type=str, action="store")
    parser_gen.add_argument("--from", type=str, nargs="+", action="store", required=True, dest="src")
    return parser

def _add_table_arguments(parser):
    parser.add_argument("-c", "--columns", action="store", nargs="*", default=[])
    parser.add_argument("identifiers", action="store", nargs="*", default=[])
    parser.add_argument(
        "-f", "--format",
        action="store",
        type=str, choices=["default", "fwf"] + [
            method.lstrip("to_")
            for method
            in dir(DataFrame)
            if method.startswith("to_")
        ],
        default="default",
    )
    parser.add_argument(
        "-t", "--tablefmt",
        action="store",
        type=str,
        metavar="TABULAR_TABLEFMT",
        default="plain",
    )


def _validate_columns(requested, existing):
    def force_column(lst, col):
        if col in existing:
            if col in lst:
                return
            lst.insert(0, col)
    columns = requested.copy()
    force_column(columns, "date")
    force_column(columns, "bname")
    wrong_columns = list(filter(lambda x: x not in existing, columns))
    if wrong_columns:
        warnings.warn(
            f"Column(s) {', '.join(wrong_columns)} skipped."
        )
        columns = [column for column in columns if column not in wrong_columns]
    return columns

def _table_service(args):
    raise NotImplementedError
    if args.columns:
        columns = _validate_columns(args.columns, df.columns)
        df = df[columns]
    identifiers = args.identifiers
    fmt = args.format
    if fmt == "default":
        print(df)
        return
    if fmt == "fwf":
        print(
            tabulate(
                df.values.tolist(),
                list(df.columns),
                tablefmt=args.tablefmt
            )
        )
        return
    if hasattr(df, f"to_{fmt}"):
        print(getattr(df, f"to_{fmt}")())
        return
    raise ValueError(
        f"Unknown format: '{fmt}'."
    )

def _list_service(args):
    identifier = args.identifier
    summarize = args.summarize
    identifiers = get_request_identifiers(month=identifier, subrequests=True)
    if not summarize:
        print("\n".join(identifiers))
        return
    identifier_ns = get_request_identifiers(month=identifier, subrequests=False)
    for i, request in enumerate(identifier_ns):
        if request not in identifiers:
            identifiers = list(filter(
                lambda x: not str.startswith(x, request),
                identifiers
            ))
            identifiers.insert(i, f"{request}.*")
    print("\n".join(identifiers))

def _column_service(args):
    raise NotImplementedError
    print("\n".join(sorted(df.columns)))

def _generate_service(args):
    generate_requests(args.identifier)

def _template_service(args):
    create_template(src=args.src, dst=args.dst)




_AVAILABLE_SERVICES = {
    "show": _table_service,
    "ls": _list_service,
    "col": _column_service,
    "gen": _generate_service,
    "template": _template_service,
}

def req(argv=None):
    if argv is None:
        argv = sys.argv[1:]

    parser = _get_parser()
    args = parser.parse_args(argv)
    service = args.service
    if not service:
        parser.print_help()
        sys.exit(1)
    _AVAILABLE_SERVICES[service](args)

